#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/* a Hogwartian student */
struct student {
    char name[100];
    double qbrv, qded, qintl, qambn;
};

/* sort functions for qsort */
int sort_name(const void* ap, const void* bp) {
    struct student* a = (struct student*) ap;
    struct student* b = (struct student*) bp;
    return strcmp(a->name, b->name);
}

/* other four are generated via macro metaprogramming
 * this is why I like C :)
 */
#define GEN_SORT(quality) \
    int sort_##quality(const void* ap, const void* bp) { \
    struct student* a = (struct student*) ap; \
    struct student* b = (struct student*) bp; \
        if (a ->q##quality > b ->q##quality) return -1; \
        else if (a->q##quality < b->q##quality) return 1; \
        else return strcmp(a->name, b->name); \
    }
GEN_SORT(brv)
GEN_SORT(ded)
GEN_SORT(intl)
GEN_SORT(ambn)

/* read the file into the array */
void read(int n, struct student* pop) {
    int ii;
    for (ii = 0; ii < n; ii++) {
        int b, d, i, a;
        scanf("%s %d %d %d %d\n", pop[ii].name, &b, &d, &i, &a);
        double total = b + d + i + a;
        pop[ii].qbrv = b / total;
        pop[ii].qded = d / total;
        pop[ii].qintl = i / total;
        pop[ii].qambn = a / total;
    }
}

/* print all students in list */
void report(int n, struct student* pop) {
    int i;
    qsort(pop, n, sizeof(struct student), &sort_name); 
    for (i = 0; i < n; i++) {
        printf("%s\n", pop[i].name);
    }
    printf("\n");
}

int main() {
    /* figure out how many */
    int n, i;
    scanf("%d\n", &n);
    struct student* pop = malloc(n * sizeof(struct student));

    /* read them in */
    read(n, pop);

    /* create house arrays */
    int ngryffindor = 0, nhufflepuff = 0, nravenclaw = 0, nslytherin = 0;
    struct student* gryffindor = malloc(n * sizeof(struct student));
    struct student* hufflepuff = malloc(n * sizeof(struct student));
    struct student* ravenclaw = malloc(n * sizeof(struct student));
    struct student* slytherin = malloc(n * sizeof(struct student));

    /* do the draft */
    int turn = 0;
    while (n > 0) {
        /* switch on the turn it is, and sort based on the right criteria */
        int* nhouse;
        struct student* house;
        switch (turn & 3) {
            case 0:
                house = gryffindor;
                nhouse = &ngryffindor;
                qsort(pop, n, sizeof(struct student), &sort_brv); 
                break;
            case 1:
                house = hufflepuff;
                nhouse = &nhufflepuff;
                qsort(pop, n, sizeof(struct student), &sort_ded); 
                break;
            case 2:
                house = ravenclaw;
                nhouse = &nravenclaw;
                qsort(pop, n, sizeof(struct student), &sort_intl); 
                break;
            case 3:
                house = slytherin;
                nhouse = &nslytherin;
                qsort(pop, n, sizeof(struct student), &sort_ambn); 
                break;
        }

        /* take the student in slot 0 into this house */
        house[(*nhouse)++] = pop[0];
        
        /* overwrite this person with last in array, and move on */
        pop[0] = pop[n - 1];
        turn++;
        n--;
    }

    /* dump all the data */
    printf("Gryffindor:\n");
    report(ngryffindor, gryffindor);
    printf("Hufflepuff:\n");
    report(nhufflepuff, hufflepuff);
    printf("Ravenclaw:\n");
    report(nravenclaw, ravenclaw);
    printf("Slytherin:\n");
    report(nslytherin, slytherin);

    /* free all the memory used */
    free(pop);
    free(gryffindor);
    free(hufflepuff);
    free(ravenclaw);
    free(slytherin);

    return 0;
}

