#include <stdio.h>
#include <stdlib.h>

/* the board */
struct board {
    int w, h;
    char* grid;
};

/* return the cell of a given board, or WALL if out of bounds */
char cell(struct board* b, int row, int col) {
    if (row < 0 || row >= b->h || col < 0 || col > b->w) {
        return '#';
    }
    return b->grid[row * b->w + col];
}

/* set a cell on the board */
void set_cell(struct board* b, int row, int col, char new) {
    b->grid[row * b->w + col] = new;
}

/* read a baord from stdin */
struct board* read_board() {
    struct board* b = malloc(sizeof (struct board));
    scanf("%d", &b->w);
    scanf("%d", &b->h);
    scanf("\n");

    b->grid = malloc(b->w * b->h * sizeof(char));
    int row, col;
    for (row = 0; row < b->h; row++) {
        for (col = 0; col < b->w; col++) {
            scanf("%c", b->grid + row*b->w + col);
        }
        scanf("\n");
    }
    
    return b;
}

/* dump a board for debugging */
void dump_board(struct board* b) {
    int row, col;
    for (row = 0; row < b->h; row++) {
        for (col = 0; col < b->w; col++) {
            printf("%c", cell(b, row, col));
        }
        printf("\n");
    }
}

/* free memory allocated to board */
void free_board(struct board* b) {
    free(b->grid);
    free(b);
}

/* do dfs at a given point */
int dfsp(struct board* b, int hrow, int hcol, int srow, int scol) {
    /* check if we're done */
    if (hrow == srow && hcol == scol) {
        return 1;
    }

    /* mark this spot as seen */
    set_cell(b, hrow, hcol, 'X');

    /* if we can go left, search there */
    if (cell(b, hrow, hcol - 1) == ' ')
        if (dfsp(b, hrow, hcol - 1, srow, scol))
            return 1;

    /* if we can go right, search there */
    if (cell(b, hrow, hcol + 1) == ' ')
        if (dfsp(b, hrow, hcol + 1, srow, scol))
            return 1;

    /* if we can go up, search there */
    if (cell(b, hrow - 1, hcol) == ' ')
        if (dfsp(b, hrow - 1, hcol, srow, scol))
            return 1;

    /* if we can go down, search there */
    if (cell(b, hrow + 1, hcol) == ' ')
        if (dfsp(b, hrow + 1, hcol, srow, scol))
            return 1;

    /* if we get all the way down here, we failed :( */
    return 0;
}


/* do a dfs from harry to sword */
int dfs(struct board* b) {
    /* start by finding harry and sword */
    int row, col, harry_row, harry_col, target_row, target_col;
    for (row = 0; row < b->h; row++) {
        for (col = 0; col < b->w; col++) {
            if (cell(b, row, col) == 'H') {
                harry_row = row;
                harry_col = col;
            }
            if (cell(b, row, col) == 'S') {
                target_row = row;
                target_col = col;
                set_cell(b, row, col, ' ');
            }
        }
    }

    /* start the recursive process here */
    return dfsp(b, harry_row, harry_col, target_row, target_col);
}

/* the whole thang */
int main() {
    struct board* b = read_board();

    /* check if harry can get to the sword */
    int can = dfs(b);

    printf("Harry can %sreach the stairs!\n", can ? "" : "not ");

    free_board(b);
    return 0;
}

